@@ -558,7 +558,7 @@ static void
 nautilus_window_constructed (GObject *self)
 {
 	NautilusWindow *window;
-	GtkWidget *table;
+	GtkWidget *grid;
 	GtkWidget *menu;
 	GtkWidget *statusbar;
 	GtkWidget *hpaned;
@@ -572,10 +572,10 @@ nautilus_window_constructed (GObject *self)
 
 	G_OBJECT_CLASS (nautilus_window_parent_class)->constructed (self);
 
-	table = gtk_table_new (1, 6, FALSE);
-	window->details->table = table;
-	gtk_widget_show (table);
-	gtk_container_add (GTK_CONTAINER (window), table);
+	grid = gtk_grid_new ();
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
+	gtk_widget_show (grid);
+	gtk_container_add (GTK_CONTAINER (window), grid);
 
 	statusbar = gtk_statusbar_new ();
 	window->details->statusbar = statusbar;
@@ -588,25 +588,19 @@ nautilus_window_constructed (GObject *self)
 
 	menu = gtk_ui_manager_get_widget (window->details->ui_manager, "/MenuBar");
 	window->details->menubar = menu;
+	gtk_widget_set_hexpand (menu, TRUE);
 	gtk_widget_show (menu);
-	gtk_table_attach (GTK_TABLE (table),
-			  menu, 
-			  /* X direction */                   /* Y direction */
-			  0, 1,                               0, 1,
-			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0,
-			  0,                                  0);
+	gtk_container_add (GTK_CONTAINER (grid), menu);
 
 	/* Register to menu provider extension signal managing menu updates */
 	g_signal_connect_object (nautilus_signaller_get_current (), "popup_menu_changed",
 			 G_CALLBACK (nautilus_window_load_extension_menus), window, G_CONNECT_SWAPPED);
 
 	window->details->content_paned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
-	gtk_table_attach (GTK_TABLE (window->details->table),
-			  window->details->content_paned,
-			  /* X direction */                   /* Y direction */
-			  0, 1,                               3, 4,
-			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, GTK_EXPAND | GTK_FILL | GTK_SHRINK,
-			  0,                                  0);
+	gtk_widget_set_hexpand (window->details->content_paned, TRUE);
+	gtk_widget_set_vexpand (window->details->content_paned, TRUE);
+
+	gtk_container_add (GTK_CONTAINER (grid), window->details->content_paned);
 	gtk_widget_show (window->details->content_paned);
 
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
@@ -632,8 +626,7 @@ nautilus_window_constructed (GObject *self)
 	pane = nautilus_window_pane_new (window);
 	window->details->panes = g_list_prepend (window->details->panes, pane);
 
-	gtk_paned_pack1 (GTK_PANED (hpaned), pane->widget, TRUE, FALSE);
-	gtk_widget_show (pane->widget);
+	gtk_paned_pack1 (GTK_PANED (hpaned), GTK_WIDGET (pane), TRUE, FALSE);
 
 	/* this has to be done after the location bar has been set up,
 	 * but before menu stuff is being called */
@@ -700,6 +693,16 @@ free_stored_viewers (NautilusWindow *window)
 	window->details->extra_viewer = NULL;
 }
 
+static void
+destroy_panes_foreach (gpointer data,
+		       gpointer user_data)
+{
+	NautilusWindowPane *pane = data;
+	NautilusWindow *window = user_data;
+
+	nautilus_window_close_pane (window, pane);
+}
+
 static void
 nautilus_window_destroy (GtkWidget *object)
 {
@@ -708,12 +711,14 @@ nautilus_window_destroy (GtkWidget *object)
 
 	window = NAUTILUS_WINDOW (object);
 
+	DEBUG ("Destroying window");
+
 	window->details->content_paned = NULL;
 	window->details->split_view_hpane = NULL;
 
 	/* close all panes safely */
 	panes_copy = g_list_copy (window->details->panes);
-	g_list_foreach (panes_copy, (GFunc) nautilus_window_close_pane, NULL);
+	g_list_foreach (panes_copy, (GFunc) destroy_panes_foreach, window);
 	g_list_free (panes_copy);
 
 	/* the panes list should now be empty */
@@ -735,24 +740,20 @@ nautilus_window_finalize (GObject *object)
 		window->details->sidebar_width_handler_id = 0;
 	}
 
-	g_free (window->details->sidebar_id);
-	g_clear_object (&window->details->nav_state);
-
-	nautilus_window_remove_trash_monitor_callback (window);
-	free_stored_viewers (window);
-
-	if (window->details->bookmark_list != NULL) {
-		g_object_unref (window->details->bookmark_list);
-	}
-
+	nautilus_window_finalize_menus (window);
 	g_signal_handlers_disconnect_by_func (nautilus_window_state,
 					      side_pane_id_changed, window);
 
+	g_clear_object (&window->details->nav_state);
+	g_clear_object (&window->details->bookmark_list);
+	g_clear_object (&window->details->ui_manager);
+
+	g_free (window->details->sidebar_id);
+	free_stored_viewers (window);
+
 	/* nautilus_window_close() should have run */
 	g_assert (window->details->panes == NULL);
 
-	g_object_unref (window->details->ui_manager);
-
 	G_OBJECT_CLASS (nautilus_window_parent_class)->finalize (object);
 }
 
@@ -796,10 +797,9 @@ nautilus_window_view_visible (NautilusWindow *window,
 	}
 
 	slot->visible = TRUE;
-
 	pane = slot->pane;
 
-	if (pane->visible) {
+	if (gtk_widget_get_visible (GTK_WIDGET (pane))) {
 		return;
 	}
 
@@ -813,13 +813,13 @@ nautilus_window_view_visible (NautilusWindow *window,
 	}
 
 	/* None, this pane is visible */
-	nautilus_window_pane_show (pane);
+	gtk_widget_show (GTK_WIDGET (pane));
 
 	/* Look for other non-visible panes */
 	for (walk = window->details->panes; walk; walk = walk->next) {
 		pane = walk->data;
 
-		if (!pane->visible) {
+		if (!gtk_widget_get_visible (GTK_WIDGET (pane))) {
 			return;
 		}
 	}
@@ -883,13 +883,10 @@ nautilus_window_open_slot (NautilusWindowPane *pane,
 }
 
 void
-nautilus_window_close_pane (NautilusWindowPane *pane)
+nautilus_window_close_pane (NautilusWindow *window,
+			    NautilusWindowPane *pane)
 {
-	NautilusWindow *window;
-
 	g_assert (NAUTILUS_IS_WINDOW_PANE (pane));
-	g_assert (NAUTILUS_IS_WINDOW (pane->window));
-	g_assert (g_list_find (pane->window->details->panes, pane) != NULL);
 
 	while (pane->slots != NULL) {
 		NautilusWindowSlot *slot = pane->slots->data;
@@ -897,10 +894,8 @@ nautilus_window_close_pane (NautilusWindowPane *pane)
 		nautilus_window_close_slot (slot);
 	}
 
-	window = pane->window;
-
 	/* If the pane was active, set it to NULL. The caller is responsible
-	 * for setting a new active pane with nautilus_window_pane_switch_to()
+	 * for setting a new active pane with nautilus_window_set_active_pane()
 	 * if it wants to continue using the window. */
 	if (window->details->active_pane == pane) {
 		window->details->active_pane = NULL;
@@ -908,7 +903,7 @@ nautilus_window_close_pane (NautilusWindowPane *pane)
 
 	window->details->panes = g_list_remove (window->details->panes, pane);
 
-	g_object_unref (pane);
+	gtk_widget_destroy (GTK_WIDGET (pane));
 }
 
 void
@@ -922,6 +917,8 @@ nautilus_window_close_slot (NautilusWindowSlot *slot)
 	g_assert (NAUTILUS_IS_WINDOW_PANE(slot->pane));
 	g_assert (g_list_find (slot->pane->slots, slot) != NULL);
 
+	DEBUG ("Closing slot %p", slot);
+
 	/* save pane because slot is not valid anymore after this call */
 	pane = slot->pane;
 	notebook = GTK_NOTEBOOK (pane->notebook);
@@ -972,6 +969,9 @@ nautilus_window_set_active_pane (NautilusWindow *window,
 				 NautilusWindowPane *new_pane)
 {
 	g_assert (NAUTILUS_IS_WINDOW_PANE (new_pane));
+
+	DEBUG ("Setting new pane %p as active", new_pane);
+
 	if (new_pane->active_slot) {
 		nautilus_window_set_active_slot (window, new_pane->active_slot);
 	} else if (new_pane != window->details->active_pane) {
@@ -989,6 +989,8 @@ nautilus_window_set_active_slot (NautilusWindow *window, NautilusWindowSlot *new
 
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
+	DEBUG ("Setting new slot %p as active", new_slot);
+
 	if (new_slot) {
 		g_assert (NAUTILUS_IS_WINDOW_SLOT (new_slot));
 		g_assert (NAUTILUS_IS_WINDOW_PANE (new_slot->pane));
@@ -1091,26 +1093,25 @@ static gboolean
 nautilus_window_key_press_event (GtkWidget *widget,
 				 GdkEventKey *event)
 {
-	static gpointer grand_parent_class = NULL;
 	NautilusWindow *window;
+	NautilusWindowSlot *active_slot;
+	NautilusView *view;
 	int i;
-	gboolean handled;
 
 	window = NAUTILUS_WINDOW (widget);
-	handled = FALSE;
 
-	if (!grand_parent_class) {
-		grand_parent_class = g_type_class_peek_parent (nautilus_window_parent_class);
-	}
+	active_slot = nautilus_window_get_active_slot (window);
+	view = active_slot->content_view;
 
-	/* See https://bugzilla.gnome.org/show_bug.cgi?id=314431
-	 *
-	 * We need to handle the keybindings for the currently focused widget
-	 * before the common accelerator bindings.
-	 */
-	handled = gtk_window_propagate_key_event (GTK_WINDOW (window), event);
-	if (handled) {
-		return TRUE;
+	if (view != NULL && nautilus_view_get_is_renaming (view)) {
+		/* if we're renaming, just forward the event to the
+		 * focused widget and return. We don't want to process the window
+		 * accelerator bindings, as they might conflict with the 
+		 * editable widget bindings.
+		 */
+		if (gtk_window_propagate_key_event (GTK_WINDOW (window), event)) {
+			return TRUE;
+		}
 	}
 
 	for (i = 0; i < G_N_ELEMENTS (extra_window_keybindings); i++) {
@@ -1129,22 +1130,14 @@ nautilus_window_key_press_event (GtkWidget *widget,
 			g_assert (action != NULL);
 			if (gtk_action_is_sensitive (action)) {
 				gtk_action_activate (action);
-				handled = TRUE;
+				return TRUE;
 			}
 
 			break;
 		}
 	}
 
-	if (!handled) {
-		handled  = gtk_window_activate_key (GTK_WINDOW (window), event);
-	}
-
-	if (!handled) {
-		handled = GTK_WIDGET_CLASS (grand_parent_class)->key_press_event (widget, event);
-	}
-
-	return handled;
+	return GTK_WIDGET_CLASS (nautilus_window_parent_class)->key_press_event (widget, event);
 }
 
 /*
@@ -1854,9 +1847,9 @@ create_extra_pane (NautilusWindow *window)
 
 	paned = GTK_PANED (window->details->split_view_hpane);
 	if (gtk_paned_get_child1 (paned) == NULL) {
-		gtk_paned_pack1 (paned, pane->widget, TRUE, FALSE);
+		gtk_paned_pack1 (paned, GTK_WIDGET (pane), TRUE, FALSE);
 	} else {
-		gtk_paned_pack2 (paned, pane->widget, TRUE, FALSE);
+		gtk_paned_pack2 (paned, GTK_WIDGET (pane), TRUE, FALSE);
 	}
 
 	/* slot */
@@ -2195,7 +2188,7 @@ nautilus_window_split_view_off (NautilusWindow *window)
 		next = l->next;
 		pane = l->data;
 		if (pane != active_pane) {
-			nautilus_window_close_pane (pane);
+			nautilus_window_close_pane (window, pane);
 		}
 	}
 
